/*
 * A tool to program our soft-core (uRV and LM32) within the SPEC.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <getopt.h>

#include "speclib.h"

static char git_version[] = "version: " GIT_VERSION;

static void print_version(char *pname)
{
	printf("%s %s\n", pname, git_version);
	printf("%s\n", libspec_version_s);
}

enum cpu_type {
    unknown = 0,
    lm32,
    riscv
};

int main(int argc, char **argv)
{
	int bus = -1, dev_fn = -1, c;
	uint32_t lm32_base = 0x80000;
	uint32_t urv_csr_base = 0x80000;
	uint32_t cpu_base = 0x0;
	enum cpu_type cpu_type = lm32; /* make it backward compatible */
	void *card;
	int read_mode = 0;
	size_t read_size = 0;


	while ((c = getopt (argc, argv, "b:d:c:ilrs:V")) != -1)
	{
		switch(c)
		{
		case 'b':
			sscanf(optarg, "%i", &bus);
			break;
		case 'd':
			sscanf(optarg, "%i", &dev_fn);
			break;
		case 'c':
			sscanf(optarg, "%i", &cpu_base);
			break;
		case 'i':
			/* RISCV mode */
			cpu_type = riscv;
			break;
		case 'l':
			/* LM32 mode */
			cpu_type = lm32;
			break;
		case 'r':
			read_mode = 1;
			break;
		case 's':
			/* read size */
			sscanf(optarg, "%li", &read_size);
			break;
		case 'V':
			print_version(argv[0]);
			exit(0);
		default:
			fprintf(stderr,
				"Use: \"%s [-V] [-b bus] [-d devfn] "
				"[-c <base address>] [-l|-s] "
				"<program.bin>\"\n"
				"-l - load into LM32 memory (default)\n"
				"-i - load into uRV(RISCV)\n"
				"-r - read memory from uRV(RISCV) into a file <program.bin>\n"
				"     NOTE: It will trigger the restart of the uRV core after the read!\n"
				"-s <size>\n",
				argv[0]);
			fprintf(stderr,
				"%s loads the binary into LM32 or uRV(RISCV) "
				"memory\n"
				"By default, the first available SPEC is used "
				"and the LM32 mode is assumed. The default "
				"load address for the LM32 is 0x%x, for the "
				"uRV CPU CSR is 0x%x.\n",
				argv[0], lm32_base, urv_csr_base);
			exit(1);
		}
	}

	if (!cpu_base) {
		/* Custom address not specified, use the default
		 * depending on the CPU type */
		if (cpu_type == lm32)
			cpu_base = lm32_base;
		if (cpu_type == riscv)
			cpu_base = urv_csr_base;
	}

	if (optind >= argc) {
		fprintf(stderr, "%s: Expected binary name after options.\n",
			argv[0]);
		exit(1);
	}

	card = spec_open(bus, dev_fn);
	if(!card)
	{
		fprintf(stderr, "%s: Can't detect a SPEC card under the given "
			"adress.\nMake sure a SPEC card is present in your PC, "
			"the driver is loaded and you run the program as root.\n", argv[0]);
		exit(1);
	}

	if (read_mode) {
		if (cpu_type != riscv) {
			fprintf(stderr, "%s: read of memory (-r parameter) can"
				" be used only for RISCV architecture. "
				"Exiting...\n", argv[0]);
			exit(0);
		}

		spec_dump_urv_mem(card, argv[optind], urv_csr_base, read_size);
		exit(0);
	}

	if (cpu_type == lm32
	    && spec_load_lm32(card, argv[optind], urv_csr_base) < 0) {
		fprintf(stderr, "%s: LM32 Loader failure.\n", argv[0]);
		exit(1);
	}

	if (cpu_type == riscv
	    && spec_load_urv(card, argv[optind], urv_csr_base) < 0) {
		fprintf(stderr, "%s: RISCV Loader failure.\n", argv[0]);
		exit(1);
	}

	spec_close(card);

	exit (0);
}
